@@ -19,7 +19,6 @@
 
 #include "parser/parser_factory.h"
 
-#include "gui/userdefdialog/userdefdialog.h"
 #include "gui/editor/schematicscene.h"
 #include "gui/editor/schematicsceneparser.h"
 #include "gui/editor/undoredocommand.h"
@@ -28,24 +27,28 @@
 #include "gui/editor/wire.h"
 #include "gui/editor/item.h"
 
+#include "gui/settings.h"
 #include "gui/qlogger.h"
 
 #include "gui/qtsolutions/qtpropertyeditor/QtBoolPropertyManager"
 #include "gui/qtsolutions/qtpropertyeditor/QtGroupPropertyManager"
 #include "gui/qtsolutions/qtpropertyeditor/QtStringPropertyManager"
-#include "gui/qtsolutions/qtpropertyeditor/QtDoublePropertyManager"
 
 #include "gui/qtsolutions/qtpropertyeditor/QtLineEditFactory"
 #include "gui/qtsolutions/qtpropertyeditor/QtCheckBoxFactory"
-#include "gui/qtsolutions/qtpropertyeditor/QtDoubleSpinBoxFactory"
 
 #include "gui/qtsolutions/qtpropertyeditor/QtAbstractPropertyBrowser"
 #include "gui/qtsolutions/qtpropertyeditor/QtProperty"
 
 #include <QtCore/QEvent>
+#include <QtCore/QRegExp>
+#include <QtCore/QCryptographicHash>
+#include <QtCore/QPointer>
+
 #include <QtGui/QKeyEvent>
 #include <QtGui/QMenu>
 #include <QtGui/QPainter>
+#include <QtGui/QFileDialog>
 #include <QtGui/QUndoCommand>
 #include <QtGui/QGraphicsView>
 #include <QtGui/QGraphicsSceneMouseEvent>
@@ -55,6 +58,7 @@
 #include <QtGui/QClipboard>
 
 #include <sstream>
+#include <memory>
 
 
 namespace qsapecng
@@ -153,6 +157,15 @@ QPainterPath SchematicScene::voltmeterPath()
   path.lineTo(step, 3 * step);
   path.lineTo(step + step / 2, 2 * step);
 
+  path.moveTo(step + step / 2, step / 2 - step / 4);
+  path.lineTo(step + step / 2, step / 2 + step / 4);
+
+  path.moveTo(step + step / 2 - step / 4, step / 2);
+  path.lineTo(step + step / 2 + step / 4, step / 2);
+
+  path.moveTo(step + step / 2 - step / 4, 4 * step + step / 2);
+  path.lineTo(step + step / 2 + step / 4, 4 * step + step / 2);
+
   return path;
 }
 
@@ -180,6 +193,13 @@ QPainterPath SchematicScene::ammeterPath()
   path.moveTo(step + step / 4, 2 * step + step / 2);
   path.lineTo(step - step / 4, 2 * step + step / 2);
 
+  path.moveTo(step / 2, 5 * step / 4);
+  path.lineTo(step / 2, step / 4);
+  path.moveTo(step / 2 - step / 4, 2 * step / 4);
+  path.lineTo(step / 2, step / 4);
+  path.moveTo(step / 2 + step / 4, 2 * step / 4);
+  path.lineTo(step / 2, step / 4);
+
   return path;
 }
 
@@ -215,8 +235,8 @@ QPainterPath SchematicScene::cccsPath()
   path.lineTo(2 * step, 3 * step + step / 2);
   path.lineTo(3 * step, 2 * step + step / 2);
   path.lineTo(2 * step, step + step / 2);
-  path.moveTo(2 * step, 3 * step + step / 2);
 
+  path.moveTo(2 * step, 3 * step + step / 2);
   path.lineTo(2 * step, 5 * step);
   path.lineTo(0, 5 * step);
 
@@ -257,8 +277,8 @@ QPainterPath SchematicScene::ccvsPath()
   path.lineTo(2 * step, 3 * step + step / 2);
   path.lineTo(3 * step, 2 * step + step / 2);
   path.lineTo(2 * step, step + step / 2);
-  path.moveTo(2 * step, 3 * step + step / 2);
 
+  path.moveTo(2 * step, 3 * step + step / 2);
   path.lineTo(2 * step, 5 * step);
   path.lineTo(0, 5 * step);
 
@@ -276,11 +296,17 @@ QPainterPath SchematicScene::ccvsPath()
   path.lineTo(4 * step, 5 * step);
   path.lineTo(5 * step, 5 * step);
 
-  path.moveTo(4 * step - step / 3, 2 * step + step / 4);
-  path.lineTo(4 * step, step - step / 4);
+  // path.moveTo(4 * step - step / 3, 2 * step + step / 4);
+  // path.lineTo(4 * step, step - step / 4);
 
-  path.moveTo(4 * step + step / 3, 2 * step + step / 4);
-  path.lineTo(4 * step, step - step / 4);
+  // path.moveTo(4 * step + step / 3, 2 * step + step / 4);
+  // path.lineTo(4 * step, step - step / 4);
+  
+  path.moveTo(4 * step - step / 3, 3 * step - step / 4);
+  path.lineTo(4 * step, 4 * step + step / 4);
+
+  path.moveTo(4 * step + step / 3, 3 * step - step / 4);
+  path.lineTo(4 * step, 4 * step + step / 4);
 
   return path;
 }
@@ -428,8 +454,8 @@ QPainterPath SchematicScene::vccsPath()
   path.lineTo(2 * step, 3 * step + step / 2);
   path.lineTo(3 * step, 2 * step + step / 2);
   path.lineTo(2 * step, step + step / 2);
-  path.moveTo(2 * step, 3 * step + step / 2);
 
+  path.moveTo(2 * step, 3 * step + step / 2);
   path.lineTo(2 * step, 5 * step);
   path.lineTo(0, 5 * step);
 
@@ -447,12 +473,12 @@ QPainterPath SchematicScene::vccsPath()
   path.moveTo(4 * step, 5 * step);
   path.lineTo(5 * step, 5 * step);
 
-  path.moveTo(4 * step, step - step / 4);
-  path.lineTo(4 * step, step + step / 4);
-
   path.moveTo(4 * step - step / 4, step);
   path.lineTo(4 * step + step / 4, step);
 
+  path.moveTo(4 * step, 4 * step - step / 4);
+  path.lineTo(4 * step, 4 * step + step / 4);
+
   path.moveTo(4 * step - step / 4, 4 * step);
   path.lineTo(4 * step + step / 4, 4 * step);
 
@@ -473,8 +499,8 @@ QPainterPath SchematicScene::vcvsPath()
   path.lineTo(2 * step, 3 * step + step / 2);
   path.lineTo(3 * step, 2 * step + step / 2);
   path.lineTo(2 * step, step + step / 2);
-  path.moveTo(2 * step, 3 * step + step / 2);
 
+  path.moveTo(2 * step, 3 * step + step / 2);
   path.lineTo(2 * step, 5 * step);
   path.lineTo(0, 5 * step);
 
@@ -685,8 +711,8 @@ Item* SchematicScene::itemByType(SchematicScene::SupportedItemType type)
       QList<QPointF> nodes;
       nodes.push_back(QPointF(0, 0));
       nodes.push_back(QPointF(0, 5 * GridStep));
-      nodes.push_back(QPointF(5 * GridStep, 0));
       nodes.push_back(QPointF(5 * GridStep, 5 * GridStep));
+      nodes.push_back(QPointF(5 * GridStep, 0));
       item = new Component(ccvsPath(), nodes);
       break;
     }
@@ -736,8 +762,8 @@ Item* SchematicScene::itemByType(SchematicScene::SupportedItemType type)
       QList<QPointF> nodes;
       nodes.push_back(QPointF(0, 5 * GridStep));
       nodes.push_back(QPointF(0, 0));
-      nodes.push_back(QPointF(5 * GridStep, 0));
       nodes.push_back(QPointF(5 * GridStep, 5 * GridStep));
+      nodes.push_back(QPointF(5 * GridStep, 0));
       item = new Component(vccsPath(), nodes);
       break;
     }
@@ -820,7 +846,7 @@ Item* SchematicScene::itemByType(SchematicScene::SupportedItemType type)
       item = new Label;
       break;
     }
-  case NilItemType:
+  case UserDefItemType:
     {
       QList<QPointF> nodes;
       item = new Component(nilPath(), nodes);
@@ -879,6 +905,8 @@ QString SchematicScene::itemNameByType(SchematicScene::SupportedItemType type)
     return QString(QObject::tr("CCCS"));
   case OpAmplItemType:
     return QString(QObject::tr("Operational Amplifier"));
+  case UserDefItemType:
+    return QString(QObject::tr("User Defined"));
   default:
     return QString(QObject::tr("Unknow"));
   }
@@ -917,6 +945,8 @@ QChar SchematicScene::itemIdByType(SchematicScene::SupportedItemType type)
     return QChar('F');
   case OpAmplItemType:
     return QChar('A');
+  case UserDefItemType:
+    return QChar('X');
   default:
     break;
   }
@@ -939,10 +969,14 @@ QPainterPath SchematicScene::userDefPath(uint ports)
     path.lineTo(step, 0);
   } else {
     path.moveTo(0, 0);
-    path.lineTo(0, step);
-    path.lineTo(step, step);
-    path.lineTo(step, 0);
+    path.lineTo(0, 2 * step);
+    path.lineTo(2 * step, 2 * step);
+    path.lineTo(2 * step, 0);
     path.lineTo(0, 0);
+    path.moveTo(0, 0);
+    path.lineTo(2 * step, 2 * step);
+    path.moveTo(2 * step, 0);
+    path.lineTo(0, 2 * step);
   }
 
   for(uint i = 0; i < side; ++i) {
@@ -950,8 +984,8 @@ QPainterPath SchematicScene::userDefPath(uint ports)
     path.lineTo(0, step * (2 * i + 1));
     path.lineTo(step, step * (2 * i + 1));
     path.addText(step * 4 / 3, step * (2 * i + 1) + step / 2,
-	QFont("Times", 8, QFont::Light),
-	QString::number(i + 1));
+        QFont("Times", 8, QFont::Light),
+        QString::number(i + 1));
   }
 
   side -= ports % 2;
@@ -960,8 +994,8 @@ QPainterPath SchematicScene::userDefPath(uint ports)
     path.lineTo(6 * step, step * (2 * i + 1));
     path.lineTo(5 * step, step * (2 * i + 1));
     path.addText(4 * step - step / 3, step * (2 * i + 1) + step / 2,
-	QFont("Times", 8, QFont::Light),
-	QString::number(i + side + ports % 2 + 1));
+        QFont("Times", 8, QFont::Light),
+        QString::number(i + side + ports % 2 + 1));
   }
 
   return path;
@@ -984,6 +1018,21 @@ QList<QPointF> SchematicScene::userDefNodes(uint ports)
 }
 
 
+SchematicScene::SchematicScene(QObject* parent): QGraphicsScene(parent) {
+  setupProperties();
+  init();
+}
+
+
+SchematicScene::~SchematicScene() {
+  // delete itemized sub-circuits
+  foreach(Item* item, userDefList_)
+    delete item->data(101).value< QPointer<qsapecng::SchematicScene> >();
+
+  delete undoRedoStack_;
+}
+
+
 QPointF SchematicScene::closestGridPoint(const QPointF& pos) const
 {
   QPoint point = pos.toPoint();
@@ -1019,6 +1068,7 @@ QList<Item*> SchematicScene::activeItems() const
   items.append(portList_);
   items.append(groundList_);
   items.append(standardList_);
+  items.append(userDefList_);
   items.append(wireList_);
   items.append(labelList_);
   if(out_)
@@ -1028,29 +1078,77 @@ QList<Item*> SchematicScene::activeItems() const
 }
 
 
+QByteArray SchematicScene::registerUserDef(const SchematicScene& scene)
+{
+    std::ostringstream stream;
+    sapecng::abstract_parser* parser = new SchematicSceneParser(scene);
+    sapecng::abstract_builder* builder =
+      sapecng::builder_factory::builder(sapecng::builder_factory::INFO, stream);
+    
+    if(builder)
+      parser->parse(*builder);
+    
+    QByteArray str(stream.str().c_str());
+    QByteArray md5 = QCryptographicHash::hash(str, QCryptographicHash::Md5);
+    
+    delete builder;
+    delete parser;
+    
+    if(!userDefMap_.contains(md5))
+      userDefMap_.insert(md5, stream.str());
+    
+    return md5;
+}
+
+
+std::string SchematicScene::queryUserDef(QByteArray md5)
+{
+  std::string rep;
+
+  if(!userDefMap_.contains(md5))
+    userDefMap_.value(md5);
+  
+  return rep;
+}
+
+
 void SchematicScene::setUserDefRequest()
 {
-  UserDefDialog userDefDialog;
-  int res = userDefDialog.exec();
+  if(0 == views().size())
+    return;
+  
+  QString fileName = QFileDialog::getOpenFileName(views().at(0),
+      tr("Read file"), Settings().workspace(),
+      QString("%1;;%2")
+        .arg(tr("Info files (*.info)"))
+        .arg(tr("XML files (*.xml)"))
+    );
 
-  if(res == QDialog::Accepted
-    && !userDefDialog.fileName().trimmed().isEmpty())
-  {
-    int size = userDefDialog.size();
-    QString fileName = userDefDialog.fileName();
+  if(!fileName.isEmpty()) {
+    SchematicScene* scene = new SchematicScene(this);
+    sapecng::abstract_builder* builder = new SchematicSceneBuilder(*scene);
+    
+    QFileInfo fileInfo(fileName);
+    std::ifstream in_file(QFile::encodeName(fileName));
+    sapecng::abstract_parser* parser =
+      sapecng::parser_factory::parser(
+        fileInfo.suffix().toStdString(), in_file);
 
-    resetStatus();
+    if(parser)
+      parser->parse(*builder);
+
+    delete parser;
+    delete builder;
+    
+    QByteArray md5 = registerUserDef(*scene);
+    int size = scene->size();
+
+    delete scene;
+    
+    setActiveItem(SchematicScene::UserDefItemType);
     userDefSessionRequested_ = true;
-
-    if(userDefItem_)
-      delete userDefItem_;
-
-    userDefItem_ = SchematicScene::itemByType(NilItemType);
-    static_cast<Component*>(userDefItem_)->setPath(userDefPath(size));
-    static_cast<Component*>(userDefItem_)->addNodes(userDefNodes(size));
-
-    foreach(QGraphicsView* view, views())
-      view->setCursor(Qt::CrossCursor);
+    userDefSize_ = size;
+    userDefMD5_ = md5;
   }
 }
 
@@ -1147,7 +1245,7 @@ void SchematicScene::addSupportedItem(QGraphicsItem* gItem, bool init)
             standardList_.push_back(item);
 
             QtProperty* name =
-	      stringManager_->addProperty(itemNameByType(type));
+              stringManager_->addProperty(itemNameByType(type));
 
             if(init) {
               stringManager_->setValue(name,
@@ -1156,50 +1254,58 @@ void SchematicScene::addSupportedItem(QGraphicsItem* gItem, bool init)
               stringManager_->setValue(name, tr("noname"));
             }
 
-	    item->setData(1, QVariant::fromValue<QtProperty*>(name));
-	    static_cast<Component*>(item)->label()->setProperty(name);
-	    connect(
-	      stringManager_,
-	      SIGNAL(valueChanged(QtProperty*, const QString&)),
-	      static_cast<Component*>(item)->label(),
-	      SLOT(valueChanged(QtProperty*, const QString&))
-	    );
+            item->setData(1, QVariant::fromValue<QtProperty*>(name));
+            static_cast<Component*>(item)->label()->setProperty(name);
+            connect(
+              stringManager_,
+              SIGNAL(valueChanged(QtProperty*, const QString&)),
+              static_cast<Component*>(item)->label(),
+              SLOT(valueChanged(QtProperty*, const QString&))
+            );
 
-	    if(type != OpAmplItemType) {
+            if(type != OpAmplItemType) {
               QtProperty* pS = boolManager_->addProperty(tr("Symbolic"));
               boolManager_->setValue(pS, true);
 
               QtProperty* pV =
-		doubleManager_->addProperty(
+                stringManager_->addProperty(
                     type == MutualInductanceItemType
                   ? tr("M")
                   : tr("Value")
                 );
 
-              doubleManager_->setValue(pV, 1.0);
+              stringManager_->setValue(pV, "1.0");
               if(type == CapacitorItemType
                   || type == ConductanceItemType
                   || type == InductorItemType
                   || type == ResistorItemType
                 )
               {
-                doubleManager_->setMinimum(pV, 0.0);
+                QRegExp sn("[+]?(?:0|[1-9]\\d*)(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?");
+                stringManager_->setRegExp(pV, sn);
+              } else {
+                QRegExp sn("[+\\-]?(?:0|[1-9]\\d*)(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?");
+                stringManager_->setRegExp(pV, sn);
               }
 
-	      name->addSubProperty(pS);
-	      name->addSubProperty(pV);
+              name->addSubProperty(pS);
+              name->addSubProperty(pV);
             }
 
             if(type == MutualInductanceItemType) {
               QtProperty* lpn = stringManager_->addProperty(tr("lp:name"));
-              QtProperty* lpv = doubleManager_->addProperty(tr("lp:value"));
+              QtProperty* lpv = stringManager_->addProperty(tr("lp:value"));
               QtProperty* lsn = stringManager_->addProperty(tr("ls:name"));
-              QtProperty* lsv = doubleManager_->addProperty(tr("ls:value"));
+              QtProperty* lsv = stringManager_->addProperty(tr("ls:value"));
 
               stringManager_->setValue(lpn, name->valueText() + "_Lp");
               stringManager_->setValue(lsn, name->valueText() + "_Ls");
-              doubleManager_->setValue(lpv, 1.0);
-              doubleManager_->setValue(lsv, 1.0);
+
+              QRegExp sn("[+\\-]?(?:0|[1-9]\\d*)(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?");
+              stringManager_->setRegExp(lpv, sn);
+              stringManager_->setValue(lpv, "1.0");
+              stringManager_->setRegExp(lsv, sn);
+              stringManager_->setValue(lsv, "1.0");
 
               name->addSubProperty(lpn);
               name->addSubProperty(lpv);
@@ -1207,8 +1313,43 @@ void SchematicScene::addSupportedItem(QGraphicsItem* gItem, bool init)
               name->addSubProperty(lsv);
             }
 
-            static_cast<Component*>(item)
-	      ->label()->setPlainText(stringManager_->value(name));
+            static_cast<Component*>(item)->label()->setPlainText(
+              stringManager_->value(name));
+            
+            typeRootMap_[type]->addSubProperty(name);
+            properties_->addSubProperty(typeRootMap_[type]);
+          }
+          break;
+        }
+      case UserDefItemType:
+        {
+          if(!userDefList_.contains(item)) {
+            userDefList_.push_back(item);
+
+            QtProperty* name =
+              stringManager_->addProperty(itemNameByType(type));
+
+            stringManager_->setValue(name,
+                itemIdByType(type) + QString::number(++itemCntMap_[type]));
+
+            item->setData(1, QVariant::fromValue<QtProperty*>(name));
+            static_cast<Component*>(item)->label()->setProperty(name);
+            connect(
+              stringManager_,
+              SIGNAL(valueChanged(QtProperty*, const QString&)),
+              static_cast<Component*>(item)->label(),
+              SLOT(valueChanged(QtProperty*, const QString&))
+            );
+
+            QPointer<qsapecng::SchematicScene> rep =
+              item->data(101)
+                .value< QPointer<qsapecng::SchematicScene> >();
+
+            name->addSubProperty(rep->properties());
+            rep->initializeBrowser(browser_);
+
+            static_cast<Component*>(item)->label()->setPlainText(
+              stringManager_->value(name));
 
             typeRootMap_[type]->addSubProperty(name);
             properties_->addSubProperty(typeRootMap_[type]);
@@ -1255,8 +1396,8 @@ void SchematicScene::removeSupportedItem(QGraphicsItem* gItem)
         {
           portList_.removeAll(item);
           for(QList<Item*>::size_type i = 0; i < portList_.size(); ++i)
-            static_cast<Component*>(portList_.at(i))
-              ->label()->setPlainText(QString::number(i + 1));
+            static_cast<Component*>(portList_.at(i))->label()->setPlainText(
+              QString::number(i + 1));
           break;
         }
       case OutItemType:
@@ -1304,15 +1445,29 @@ void SchematicScene::removeSupportedItem(QGraphicsItem* gItem)
       case OpAmplItemType:
         {
           if(standardList_.contains(item)) {
-	    if(itemProperties(item))
-	      typeRootMap_[type]->removeSubProperty(itemProperties(item));
-            if(typeRootMap_[type]->subProperties().size() == 0)
-              properties_->removeSubProperty(typeRootMap_[type]);
+            if(itemProperties(item)) {
+              typeRootMap_[type]->removeSubProperty(itemProperties(item));
+              if(typeRootMap_[type]->subProperties().size() == 0)
+                properties_->removeSubProperty(typeRootMap_[type]);
+            }
 
             standardList_.removeAll(item);
           }
           break;
         }
+      case UserDefItemType:
+        {
+          if(userDefList_.contains(item)) {
+            if(itemProperties(item)) {
+              typeRootMap_[type]->removeSubProperty(itemProperties(item));
+              if(typeRootMap_[type]->subProperties().size() == 0)
+                properties_->removeSubProperty(typeRootMap_[type]);
+            }
+
+            userDefList_.removeAll(item);
+          }
+          break;
+        }
       default:
         break;
       }
@@ -1354,8 +1509,8 @@ void SchematicScene::clearSupportedItem(QGraphicsItem* gItem)
         {
           portList_.removeAll(item);
           for(QList<Item*>::size_type i = 0; i < portList_.size(); ++i)
-            static_cast<Component*>(portList_.at(i))
-              ->label()->setPlainText(QString::number(i + 1));
+            static_cast<Component*>(portList_.at(i))->label()->setPlainText(
+              QString::number(i + 1));
           break;
         }
       case OutItemType:
@@ -1403,19 +1558,32 @@ void SchematicScene::clearSupportedItem(QGraphicsItem* gItem)
       case OpAmplItemType:
         {
           if(standardList_.contains(item)) {
-            standardList_.removeAll(item);
-
             QtProperty* props = itemProperties(item);
+            if(props) {
+              typeRootMap_[type]->removeSubProperty(itemProperties(item));
+              if(typeRootMap_[type]->subProperties().size() == 0)
+                properties_->removeSubProperty(typeRootMap_[type]);
 
-            typeRootMap_[type]->removeSubProperty(props);
-            if(typeRootMap_[type]->subProperties().size() == 0)
-              properties_->removeSubProperty(typeRootMap_[type]);
+              foreach(QtProperty* prop, props->subProperties())
+                delete prop;
 
-	    if(props)
-	      foreach(QtProperty* prop, props->subProperties())
-		delete prop;
+              delete props;
+            }
 
-	    delete props;
+            standardList_.removeAll(item);
+          }
+          break;
+        }
+      case UserDefItemType:
+        {
+          if(userDefList_.contains(item)) {
+            if(itemProperties(item)) {
+              typeRootMap_[type]->removeSubProperty(itemProperties(item));
+              if(typeRootMap_[type]->subProperties().size() == 0)
+                properties_->removeSubProperty(typeRootMap_[type]);
+            }
+
+            userDefList_.removeAll(item);
           }
           break;
         }
@@ -1521,6 +1689,7 @@ void SchematicScene::copySelectedItems()
     new SchematicSceneParser(*this, selectedItems());
 
   parser->parse(*out);
+  
   delete parser;
   delete out;
 
@@ -1621,25 +1790,45 @@ void SchematicScene::clearSchematicScene()
 
 void SchematicScene::initializeBrowser(QtAbstractPropertyBrowser* browser)
 {
-  browser->setFactoryForManager(boolManager_, checkBoxFactory_);
-  browser->setFactoryForManager(stringManager_, lineEditFactory_);
-  browser->setFactoryForManager(doubleManager_, spinBoxFactory_);
+  browser_ = browser;
+  browser_->setFactoryForManager(boolManager_, checkBoxFactory_);
+  browser_->setFactoryForManager(stringManager_, lineEditFactory_);
 }
 
 
-void SchematicScene::assignNodes()
+QVector<int> SchematicScene::ports() {
+  QVector<int> vector;
+  foreach(Item* item, portList_)
+    vector += static_cast<Component*>(item)->nodes();
+  
+  return vector;
+}
+
+
+void SchematicScene::assignNodes(int seed)
 {
   resetStatus();
-  int value = SchematicScene::Ground + 1;
 
   foreach(Item* item, groundList_)
     static_cast<Component*>(item)->propagate(SchematicScene::Ground);
   foreach(Item* item, wireList_)
-    value = static_cast<Wire*>(item)->propagate(value);
+    seed = static_cast<Wire*>(item)->propagate(seed);
   foreach(Item* item, standardList_)
-    value = static_cast<Component*>(item)->propagate(value);
+    seed = static_cast<Component*>(item)->propagate(seed);
+  foreach(Item* item, portList_)
+    seed = static_cast<Component*>(item)->propagate(seed);
+  foreach(Item* item, userDefList_)
+    seed = static_cast<Component*>(item)->propagate(seed);
   if(out_)
-    value = static_cast<Component*>(out_)->propagate(value);
+    seed = static_cast<Component*>(out_)->propagate(seed);
+
+  // sub-circuits propagation
+  foreach(Item* item, userDefList_) {
+    QPointer<qsapecng::SchematicScene> rep =
+      item->data(101).value< QPointer<qsapecng::SchematicScene> >();
+
+    rep->assignNodes(seed);
+  }
 }
 
 
@@ -1651,8 +1840,17 @@ void SchematicScene::resetNodes()
     static_cast<Wire*>(item)->invalidate();
   foreach(Item* item, standardList_)
     static_cast<Component*>(item)->invalidate();
+  foreach(Item* item, portList_)
+    static_cast<Component*>(item)->invalidate();
+  foreach(Item* item, userDefList_)
+    static_cast<Component*>(item)->invalidate();
   if(out_)
     static_cast<Component*>(out_)->invalidate();
+
+  // sub-circuits reset
+  foreach(Item* item, userDefList_)
+    item->data(101)
+      .value< QPointer<qsapecng::SchematicScene> >()->resetNodes();
 }
 
 
@@ -1680,8 +1878,8 @@ void SchematicScene::resetStatus()
     view->unsetCursor();
 
   userDefSessionRequested_ = false;
-  delete userDefItem_;
-  userDefItem_ = 0;
+  userDefMD5_ = QByteArray();
+  userDefSize_ = 0;
 
   wireSessionRequested_ = false;
   delete preWireItem_;
@@ -1734,7 +1932,7 @@ void SchematicScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
   lastMousePressPos_ = event->scenePos();
   resetNodes();
 
-  if(hasActiveItem()) {
+  if(hasActiveItem() || hasUserDefReq()) {
     switch(event->button())
     {
     case Qt::RightButton:
@@ -1743,15 +1941,31 @@ void SchematicScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
       return;
     case Qt::LeftButton:
     {
-      AddSupportedItem* addCommand =
-        new AddSupportedItem(this, activeItem_, item_->pos());
+      QUndoCommand* addCommand = 0;
+      
+      if(hasUserDefReq()) {
+        addCommand = new AddUserDefItem(this,
+          userDefMD5_,
+          userDefSize_,
+          userDefMap_.value(userDefMD5_),
+          item_->pos());
+      } else {
+          addCommand = new AddSupportedItem(this, activeItem_, item_->pos());
+      }
 
       undoRedoStack_->beginMacro(addCommand->text());
       undoRedoStack_->push(addCommand);
 
       uint rotFact = item_->angle() / 90;
       for(uint i = 0; i < rotFact; ++i) {
-        RotateItems* rotateCommand = new RotateItems(addCommand->item());
+        RotateItems* rotateCommand = 0;
+
+        if(hasUserDefReq()) {
+          rotateCommand = new RotateItems(static_cast<AddUserDefItem*>(addCommand)->item());
+        } else {
+          rotateCommand = new RotateItems(static_cast<AddSupportedItem*>(addCommand)->item());
+        }
+
         undoRedoStack_->push(rotateCommand);
       }
       undoRedoStack_->endMacro();
@@ -1764,29 +1978,6 @@ void SchematicScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
     }
   }
 
-  if(hasUserDefReq()) {
-    switch(event->button())
-    {
-    case Qt::RightButton:
-      userDefItem_->rotate();
-      event->accept();
-      return;
-    case Qt::LeftButton:
-    {
-      //************ TODO ************//
-      addItem(userDefItem_);
-      userDefItem_ = 0;
-      resetStatus();
-      //************ TODO ************//
-
-      event->accept();
-      return;
-    }
-    default:
-      break;
-    }
-  }
-
   if(hasPendingWire()) {
     switch(event->button())
     {
@@ -1834,18 +2025,12 @@ void SchematicScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
 
 void SchematicScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 {
-  if(hasActiveItem()) {
+  if(hasActiveItem() || hasUserDefReq()) {
     item_->setPos(closestGridPoint(event->scenePos()));
     event->accept();
     return;
   }
 
-  if(hasUserDefReq()) {
-    userDefItem_->setPos(closestGridPoint(event->scenePos()));
-    event->accept();
-    return;
-  }
-
   if(hasPendingWire()) {
     if(hasValidFromPoint_)
       setPrePostWireLine(event->scenePos());
@@ -1857,6 +2042,28 @@ void SchematicScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 }
 
 
+void SchematicScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event)
+{
+  if(!hasActiveItem() && !hasUserDefReq() && !hasPendingWire()
+      && itemAt(event->scenePos())) {
+    if(SchematicScene::itemType(itemAt(event->scenePos()))
+        == SchematicScene::UserDefItemType)
+    {
+      QGraphicsItem* item = itemAt(event->scenePos());
+      QPointer<qsapecng::SchematicScene> rep =
+        item->data(101).value< QPointer<qsapecng::SchematicScene> >();
+
+      emit(showUserDef(*rep));
+      event->accept();
+    }
+  }
+
+  if(SchematicScene::itemType(itemAt(event->scenePos()))
+      != SchematicScene::UserDefItemType)
+    QGraphicsScene::mousePressEvent(event);
+}
+
+
 void SchematicScene::contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
 {
   if(!hasActiveItem() && !hasUserDefReq() && !hasPendingWire() && contextMenu_)
@@ -1916,39 +2123,33 @@ bool SchematicScene::event(QEvent* event)
   // enter event
   if(event->type() == QEvent::Enter)
   {
-    if(hasActiveItem()) {
+    if(hasActiveItem() || hasUserDefReq()) {
       if(item_)
-	delete item_;
+        delete item_;
 
       item_ = SchematicScene::itemByType(activeItem_);
+      
+      if(hasUserDefReq()) {
+        static_cast<Component*>(item_)->setPath(userDefPath(userDefSize_));
+        static_cast<Component*>(item_)->addNodes(userDefNodes(userDefSize_));
+      }
+      
       addItem(item_);
       event->accept();
       return true;
     }
-
-    if(hasUserDefReq()) {
-      addItem(userDefItem_);
-      event->accept();
-      return true;
-    }
   }
 
   // leave event
   if(event->type() == QEvent::Leave)
   {
-    if(hasActiveItem()) {
+    if(hasActiveItem() || hasUserDefReq()) {
       removeItem(item_);
       delete item_;
       item_ = 0;
       event->accept();
       return true;
     }
-
-    if(hasUserDefReq()) {
-      removeItem(userDefItem_);
-      event->accept();
-      return true;
-    }
   }
 
   return QGraphicsScene::event(event);
@@ -1963,9 +2164,8 @@ void SchematicScene::dragEnterEvent(QGraphicsSceneDragDropEvent* event)
   }
 
   if(event->mimeData()->hasFormat("application/qsapecng-itemtype")) {
-    SupportedItemType type =
-      (SupportedItemType) event->mimeData()
-        ->data("application/qsapecng-itemtype").toInt();
+    SupportedItemType type = (SupportedItemType) event->mimeData()->data(
+      "application/qsapecng-itemtype").toInt();
 
     item_ = itemByType(type);
     if(item_) {
@@ -2055,7 +2255,8 @@ void SchematicScene::init()
   item_ = 0;
 
   userDefSessionRequested_ = false;
-  userDefItem_ = 0;
+  userDefMD5_ = QByteArray();
+  userDefSize_ = 0;
 
   wireSessionRequested_ = false;
   wireInProgress_ = 0;
@@ -2069,7 +2270,6 @@ void SchematicScene::init()
       Qt::RoundJoin
     );
 
-  setupProperties();
   connect(undoRedoStack_, SIGNAL(indexChanged(int)),
     this, SLOT(resetNodes()));
 }
@@ -2081,15 +2281,17 @@ void SchematicScene::setupProperties()
 
   boolManager_ = new QtBoolPropertyManager(this);
   stringManager_ = new QtStringPropertyManager(this);
-  doubleManager_ = new QtDoublePropertyManager(this);
+
+  connect(boolManager_, SIGNAL(propertyChanged(QtProperty*)),
+    this, SIGNAL(propertyChanged()));
+  connect(stringManager_, SIGNAL(propertyChanged(QtProperty*)),
+    this, SIGNAL(propertyChanged()));
 
   lineEditFactory_ = new QtLineEditFactory(this);
   checkBoxFactory_ = new QtCheckBoxFactory(this);
-  spinBoxFactory_ = new QtDoubleSpinBoxFactory(this);
 
   lineEditFactory_->addPropertyManager(stringManager_);
   checkBoxFactory_->addPropertyManager(boolManager_);
-  spinBoxFactory_->addPropertyManager(doubleManager_);
 
   properties_ = groupManager_->addProperty(tr("Circuit"));
 
@@ -2157,6 +2359,11 @@ void SchematicScene::setupProperties()
   typeRootMap_[MutualInductanceItemType] =
     groupManager_->addProperty(itemNameByType(MutualInductanceItemType));
 //   properties_->addSubProperty(typeRootMap_[MutualInductanceItemType]);
+
+  itemCntMap_[UserDefItemType] = 0;
+  typeRootMap_[UserDefItemType] =
+    groupManager_->addProperty(itemNameByType(UserDefItemType));
+//   properties_->addSubProperty(typeRootMap_[UserDefItemType]);
 }
 
 
