@@ -431,7 +431,7 @@ void QSapecNGWindow::print()
 
     if(editor) {
       if(QPrintDialog(&printer).exec() != QDialog::Accepted)
-	return;
+        return;
 
       QPainter painter(&printer);
       painter.setRenderHint(QPainter::Antialiasing);
@@ -458,7 +458,7 @@ void QSapecNGWindow::props()
 
     foreach(QGraphicsItem* item, items)
       if(SchematicScene::itemProperties(item))
-	browser->addProperty(SchematicScene::itemProperties(item));
+        browser->addProperty(SchematicScene::itemProperties(item));
 
     QDialog* dialog = new QDialog(this, Qt::Dialog);
     QDialogButtonBox* buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
@@ -552,20 +552,21 @@ void QSapecNGWindow::license()
   QString title = tr("License");
   QString body =
     "QSapecNG, Qt-based GUI for SapecNG<br>"
-    "Copyright (C) 2009, Michele Caini<br>"
+    "Copyright (C) 2009-2011, Michele Caini<br>"
     "<br>"
-    "This program is free software: you can redistribute it and/or modify<br>"
-    "it under the terms of the GNU General Public License as published by<br>"
-    "the Free Software Foundation, either version 3 of the License, or<br>"
+    "This program is free software: you can redistribute it and/or modify "
+    "it under the terms of the GNU General Public License as published by "
+    "the Free Software Foundation, either version 3 of the License, or "
     "(at your option) any later version.<br>"
     "<br>"
-    "This program is distributed in the hope that it will be useful,<br>"
-    "but WITHOUT ANY WARRANTY; without even the implied warranty of<br>"
-    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br>"
+    "This program is distributed in the hope that it will be useful, "
+    "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "GNU General Public License for more details.<br>"
     "<br>"
-    "You should have received a copy of the GNU General Public License<br>"
-    "along with this program.  If not, see <http://www.gnu.org/licenses/>.";
+    "You should have received a copy of the GNU General Public License "
+    "along with this program.  If not, see "
+    "<a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a>.";
 
   QMessageBox::about(this, title, body);
 }
@@ -727,33 +728,6 @@ void QSapecNGWindow::updateDocks()
 }
 
 
-SchematicEditor* QSapecNGWindow::createSchematicEditor()
-{
-  QApplication::setOverrideCursor(Qt::WaitCursor);
-
-  SchematicEditor* editor = new SchematicEditor;
-  editor->scene().initializeBrowser(treeBrowser_);
-  editor->scene().setGridVisible(toggleGridAct_->isChecked());
-  editor->scene().setContextMenu(sceneMenu_);
-
-  undoRedoGroup_->addStack(editor->scene().undoRedoStack());
-  editor->scene().undoRedoStack()->setActive(true);
-
-  connect(this, SIGNAL(toggleGrid(bool)),
-      &(editor->scene()), SLOT(setGridVisible(bool)));
-  connect(&(editor->scene()), SIGNAL(selectionChanged()),
-      this, SLOT(sceneSelectionChanged()));
-
-  editor->setAttribute(Qt::WA_DeleteOnClose);
-  editor->setWindowIcon(QIcon(":/images/grid.png"));
-  mdiArea_->addSubWindow(editor, Qt::SubWindow);
-
-  QApplication::restoreOverrideCursor();
-
-  return editor;
-}
-
-
 void QSapecNGWindow::createLogger()
 {
   policy_ = new QLogPolicy(this);
@@ -1440,6 +1414,41 @@ QMdiSubWindow* QSapecNGWindow::findEditor(const QString& fileName)
 }
 
 
+SchematicEditor* QSapecNGWindow::createSchematicEditor()
+{
+  QApplication::setOverrideCursor(Qt::WaitCursor);
+
+  SchematicEditor* editor = new SchematicEditor;
+  setupSchematicEditor(editor);
+
+  QApplication::restoreOverrideCursor();
+
+  return editor;
+}
+
+
+void QSapecNGWindow::setupSchematicEditor(SchematicEditor* editor)
+{
+  editor->scene().initializeBrowser(treeBrowser_);
+  editor->scene().setGridVisible(toggleGridAct_->isChecked());
+  editor->scene().setContextMenu(sceneMenu_);
+
+  undoRedoGroup_->addStack(editor->scene().undoRedoStack());
+  editor->scene().undoRedoStack()->setActive(true);
+
+  connect(editor, SIGNAL(stackEditor(SchematicEditor*)),
+    this, SLOT(stackEditor(SchematicEditor*)));
+  connect(&(editor->scene()), SIGNAL(selectionChanged()),
+      this, SLOT(sceneSelectionChanged()));
+  connect(this, SIGNAL(toggleGrid(bool)),
+      &(editor->scene()), SLOT(setGridVisible(bool)));
+
+  editor->setAttribute(Qt::WA_DeleteOnClose);
+  editor->setWindowIcon(QIcon(":/images/grid.png"));
+  mdiArea_->addSubWindow(editor, Qt::SubWindow);
+}
+
+
 void QSapecNGWindow::updateRecentFileList(const QString& fileName)
 {
   QStringList files = Settings().recentFiles();
@@ -1584,4 +1593,15 @@ void QSapecNGWindow::subWindowActivated(QMdiSubWindow* window)
 }
 
 
+void QSapecNGWindow::stackEditor(SchematicEditor* editor)
+{
+  setupSchematicEditor(editor);
+  central_->setCurrentWidget(mdiArea_);
+  setActiveSubWindow(editor);
+  editor->show();
+
+  statusBar()->showMessage(tr("Component loaded"), 2000);
+}
+
+
 }
