@@ -23,10 +23,13 @@
 #include "gui/functor/functor_traits.hpp"
 #include "gui/qlogger.h"
 
+#include <QtCore/QRegExp>
+#include <QtGui/QRegExpValidator>
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QFormLayout>
 #include <QtGui/QGroupBox>
+#include <QtGui/QLineEdit>
 #include <QtGui/QDoubleSpinBox>
 #include <QtGui/QPushButton>
 #include <QtGui/QLabel>
@@ -94,12 +97,12 @@ void MarkableCurve::moved(const QwtDoublePoint& pos)
     marker_->setLabel(label);
 
     Qt::Alignment H =
-	(data().x(idx) >= minXValue() + ((maxXValue() - minXValue()) / 2))
+        (data().x(idx) >= minXValue() + ((maxXValue() - minXValue()) / 2))
       ? Qt::AlignLeft
       : Qt::AlignRight
       ;
     Qt::Alignment V =
-	(data().y(idx) >= minYValue() + ((maxYValue() - minYValue()) / 2))
+        (data().y(idx) >= minYValue() + ((maxYValue() - minYValue()) / 2))
       ? Qt::AlignBottom
       : Qt::AlignTop
       ;
@@ -223,10 +226,13 @@ void WorkPlane::setData(
   {
     QLabel* left = new QLabel(QString::fromStdString(i->first));
     left->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
-    QDoubleSpinBox* right = new QDoubleSpinBox;
-    right->setMaximum(std::numeric_limits<double>::max());
-    right->setMinimum(std::numeric_limits<double>::min());
-    right->setValue(i->second);
+    
+    QLineEdit* right = new QLineEdit;
+    right->setText(QString::number(i->second));
+    QRegExp sn("[+\\-]?(?:0|[1-9]\\d*)(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?");
+    QRegExpValidator* validator = new QRegExpValidator(sn, right);
+    right->setValidator(validator);
+    
     data_->setCellWidget(row, 0, left);
     data_->setCellWidget(row, 1, right);
   }
@@ -323,7 +329,7 @@ void WorkPlane::plot(WorkPlane::F f)
       std::pair< std::vector<double>, std::vector<double> > res;
       if(!curves_[f].second) {
         QLogger::info(QObject::tr("Calculate magnitude."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         res = sapecng::magnitude(
             std::make_pair(startFreq_->value(), endFreq_->value()),
             stepFreq_->value()
@@ -360,7 +366,7 @@ void WorkPlane::plot(WorkPlane::F f)
       std::pair< std::vector<double>, std::vector<double> > res;
       if(!curves_[f].second) {
         QLogger::info(QObject::tr("Calculate phase."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         res = sapecng::phase(
             std::make_pair(startFreq_->value(), endFreq_->value()),
             stepFreq_->value()
@@ -397,7 +403,7 @@ void WorkPlane::plot(WorkPlane::F f)
       std::pair< std::vector<double>, std::vector<double> > res;
       if(!curves_[f].second) {
         QLogger::info(QObject::tr("Calculate gain."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         res = sapecng::gain(
             std::make_pair(startFreq_->value(), endFreq_->value()),
             stepFreq_->value()
@@ -434,7 +440,7 @@ void WorkPlane::plot(WorkPlane::F f)
       std::pair< std::vector<double>, std::vector<double> > res;
       if(!curves_[f].second) {
         QLogger::info(QObject::tr("Calculate loss."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         res = sapecng::loss(
             std::make_pair(startFreq_->value(), endFreq_->value()),
             stepFreq_->value()
@@ -471,7 +477,7 @@ void WorkPlane::plot(WorkPlane::F f)
       std::pair< std::vector<double>, std::vector<double> > resZ;
       if(!curves_[ZEROS].second) {
         QLogger::info(QObject::tr("Looking for zeros."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         resZ = sapecng::zeros()(num_, den_, values);
 
         unsigned int size = resZ.first.size() > resZ.second.size() ?
@@ -479,15 +485,15 @@ void WorkPlane::plot(WorkPlane::F f)
 
         for(unsigned int i = 0; i < size; ++i)
           QLogger::info(QObject::tr("Zero found near to ")
-	    + QString("(%1, %2)")
-	      .arg(QString::number(resZ.first.at(i)))
-	      .arg(QString::number(resZ.second.at(i))));
+            + QString("(%1, %2)")
+              .arg(QString::number(resZ.first.at(i)))
+              .arg(QString::number(resZ.second.at(i))));
       }
 
       std::pair< std::vector<double>, std::vector<double> > resP;
       if(!curves_[POLES].second) {
         QLogger::info(QObject::tr("Looking for poles."));
-	std::map<std::string, double> values = actValues();
+        std::map<std::string, double> values = actValues();
         resP = sapecng::poles()(num_, den_, values);
 
         unsigned int size = resP.first.size() > resP.second.size() ?
@@ -495,9 +501,9 @@ void WorkPlane::plot(WorkPlane::F f)
 
         for(unsigned int i = 0; i < size; ++i)
           QLogger::info(QObject::tr("Pole found near to ")
-	    + QString("(%1, %2)")
-	      .arg(QString::number(resP.first.at(i)))
-	      .arg(QString::number(resP.second.at(i))));
+            + QString("(%1, %2)")
+              .arg(QString::number(resP.first.at(i)))
+              .arg(QString::number(resP.second.at(i))));
       }
 
       plot_->setTitle(
@@ -552,8 +558,9 @@ std::map<std::string, double> WorkPlane::actValues() const
 
   for(int i = 0; i < data_->rowCount(); ++i)
     values[qobject_cast<const QLabel*>(
-	data_->cellWidget(i, 0))->text().toStdString()] =
-      qobject_cast<const QDoubleSpinBox*>(data_->cellWidget(i, 1))->value();
+        data_->cellWidget(i, 0))->text().toStdString()] =
+      qobject_cast<const QLineEdit*>(
+	data_->cellWidget(i, 1))->text().toDouble();
 
   return values;
 }
